from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from yta_general_utils.experimental.chrome_scrapper import start_chrome, go_to_and_wait_loaded
from random import randrange

import time
import glob
import shutil
import os

URL = 'https://tuna.voicemod.net/text-to-song/?utm_source=LandingTTSong&utm_medium=LandingTTSong&utm_campaign=TTSongCreateLanding&utm_id=TTSongCreateLanding'
DOWNLOADS_ABSOLUTE_PATH = 'C:/Users/dania/Downloads/'
PROJECT_ABSOLUTE_PATH = os.getenv('PROJECT_ABSOLUTE_PATH')

def generate_song(lyrics, output_filename):
    try:
        cookies = [
            {
                'name': 'vm_userId',
                'value': '06755a97-1896-4856-9709-8203d1c1befc'
            },
            {
                'name': 'session',
                'value': '236gerx18e3xb82k3m81xcvnporuxysqaeqrrrs97nqp8gvv1fxspb0q3v5q4z3pnbsv1w6',
            },
            {
                'name': 'vm_refresh_token',
                'value': 'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpYXQiOjE3MDk0ODgwMzksImV4cCI6MTcxMjA4MDAzOSwic3ViIjoiMDY3NTVhOTctMTg5Ni00ODU2LTk3MDktODIwM2QxYzFiZWZjIiwiYXVkIjoid2ViIiwic2NvcGUiOiJyZWFkIHdyaXRlIn0.NWfF78CNbBQakZyofsWEYN4B7BDzGrYhZtFVPblrKNdRtCaQ6OZEvoTJGkjsbklUfzYzEcU71lqZWAdktAjjLgnEGwHKkNT0gnqm5wyw2m-VmX9K1FJcvYxrDuLSGgj4cXVknewmN7rJmNvV9m0VsI46bNX09Pd0lPcAevqouyb2G2W0yEyIyUNHxuPTrhXiBt15xIh7g7yffRJk33gRVOzwcu9xNutk32Ml-zHPcY1YhSEjXYRtDPdAmOzfXy1vNJSpI5T2BEH4NtZuegs2V3CRLoSCKFWd5mZKA3NKrEsfQjKnLIRE27VvNpNVM6PXltuI8rEgxeVlDeF4yechJQ'
            },
            {
                'name': 'vm_mparticle_device_id',
                'value': 'd533e2bc-9f88-4b47-f5bd-aa16779a6b26'
            },
            {
                'name': 'vm_initial_referrer',
                'value': 'aHR0cHM6Ly9hY2NvdW50LnZvaWNlbW9kLm5ldC8='
            },
            {
                'name': 'vm_initial_url',
                'value': 'aHR0cHM6Ly93d3cudm9pY2Vtb2QubmV0Lw=='
            },
            {
                'name': 'vm_access_token',
                'value': 'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpYXQiOjE3MDk0ODgwMzksImV4cCI6MTcwOTU3NDQzOSwic3ViIjoiMDY3NTVhOTctMTg5Ni00ODU2LTk3MDktODIwM2QxYzFiZWZjIiwiYXVkIjoid2ViIiwic2NvcGUiOiJyZWFkIHdyaXRlIn0.RVBc-3jRUlx03ulDyR7HoXW1c4ir6AdDFWqnSmMahVM2QM76zy2YY85r4n2ynvECKl5ylu7MhM4PwjyzYLISCCMgkQTkALexGJZXELBAdQTkKVhG4EuqU91VOHvYCLugaB6HkA12U9VGvbh-bgTxnRxB2dnDfOBSIUZUVx-tCNEmhpHILbKY_C4bo_UEpeC8nBLvdle2SnLZbuUNrlCIb3rFsLEdPCnAsBXeGqxQEBVKEOE-TCHwF0VvPSEo-HZo97I-yGindxpXvty2BZkbfXhurMed8Gl6UKV_rQ4deBxyl-SL1B1XFc-rzGVot_L0VyUeF10LKwK-Hwq4xzPmpQ'
            }
        ]

        driver = start_chrome(True, True, True, [])
        go_to_and_wait_loaded(driver, 'https://tuna.voicemod.net/text-to-song')
        for cookie in cookies:
            driver.add_cookie(cookie)
        go_to_and_wait_loaded(driver, URL)
        time.sleep(2)

        music_style_ids = ['DarkTrap', 'Levitate', 'LazerBeam', 'StayWithMe', 'MoveYourBody', 'BreakIsOver', 'HappyBirthday', 'Hallelujah', 'JingleBells', 'WeWishYou', 'SilentNight', 'DeckTheHalls', 'HolidayTunes', 'Joy', 'AngelsSing']
        music_style_element = driver.find_element(By.ID, music_style_ids[randrange(len(music_style_ids))])
        music_style_element.click()
        #music_style_elements = driver.find_elements(By.CLASS_NAME, 'song-preview-card')
        #music_style_elements[0].click()

        time.sleep(2)

        buttons = driver.find_elements(By.XPATH, "//button[contains(@aria-label, 'Next')]")
        buttons[0].click()

        time.sleep(2)

        music_style_elements = driver.find_elements(By.CLASS_NAME, 'step-singer-card')
        music_style_element = music_style_elements[randrange(len(music_style_elements))]
        music_style_element.click()
        #music_style_elements[0].click()

        time.sleep(2)

        buttons = driver.find_elements(By.XPATH, "//button[contains(@aria-label, 'Next')]")
        buttons[0].click()

        time.sleep(2)

        textarea = driver.find_element(By.TAG_NAME, 'textarea')
        textarea.send_keys(lyrics)

        time.sleep(2)

        buttons = driver.find_elements(By.XPATH, "//button[contains(@aria-label, 'Create song')]")
        buttons[0].click()

        time.sleep(2)

        # Check which one is the latest file
        list_of_files = glob.glob(DOWNLOADS_ABSOLUTE_PATH + '*.mp4')
        latest_file = max(list_of_files, key = os.path.getctime)

        element_present = EC.presence_of_element_located((By.XPATH, "//button[contains(@aria-label, 'Download')]"))
        element = WebDriverWait(driver, 60).until(element_present)
        # wait Button aria-label 'Download'
        element.click()
        time.sleep(2)

        # Now we need 2 ALT + SPACE
        actions = ActionChains(driver)
        for i in range(2):
            actions.send_keys(Keys.TAB)
        actions.perform()
        actions.send_keys(Keys.SPACE)
        actions.perform()

        cont = 0
        while cont < 60:
            list_of_files = glob.glob(DOWNLOADS_ABSOLUTE_PATH + '*.mp4')
            tmp_latest_file = max(list_of_files, key = os.path.getctime)
            if tmp_latest_file != latest_file:
                latest_file = tmp_latest_file
                cont = 60
            time.sleep(1)
            cont += 1

        shutil.move(latest_file, os.path.join(output_filename))
        print('Song downloaded successfully')
        
        """
        tabs_to_next = 16

        # Styles start at 5th tab
        actions = ActionChains(driver)
        for i in range(7):
            actions.send_keys(Keys.TAB)
        actions.perform()
        time.sleep(1)
        actions.send_keys(Keys.SPACE)
        actions.perform()
        time.sleep(1)

        for i in range(tabs_to_next - 7):
            actions.send_keys(Keys.TAB)
        actions.perform()
        time.sleep(1)
        actions.send_keys(Keys.SPACE)
        actions.perform()
        time.sleep(1)
        """
    except Exception as e:
        print(e)
    finally:
        driver.close()


import ollama


class Llava:
    """
    Class to describe an image using the Llava engine
    through the 'ollama' python package.
    """

    def describe(
        self,
        image_filename: str
    ):
        """
        THIS METHOD IS NOT WORKING YET.

        TODO: This is not working because of my pc limitations.
        It cannot load the resources due to memory capacity.
        """
        res = ollama.chat(
            model = 'llava',
            messages = [
                {
                    'role': 'user',
                    'content': 'Describe this image',
                    'images': [
                        image_filename
                    ]
                }
            ]
        )

        response_content = res['message']['content']

        return response_content


from ai21 import AI21Client

import os

API_KEY = os.environ.get("AI21_API_KEY")

# This AI allows you to work just with registering. They give you 90$ to start

def paraphrase(text):
    client = AI21Client(
        api_key = API_KEY,
    )

    # TODO: This model works in English, so I receive the suggestion in English

    # This method returns a lot of paraphrased, we will choose the first one
    response = client.paraphrase.create(text = text, start_index = 0)

    # Response has suggestions, with text 
    return response.suggestions[0].text

def test():
    print(paraphrase('Hola, mi nombre es Alberto y tengo el culo abierto.'))


from re_gpt import SyncChatGPT
import requests
import json


def ask_v1(prompt):
    """
    This seems to be working well.
    """
    # You go through this url (https://chat3.aiyunos.top/) and it opens one chat
    # From this chat I made the request: https://715zwv.aitianhu1.top/#/chat/1002
    # You open a new chat, make a request by web, copy cookies and thats all.
    cookies = {
        'sl-session': 'nnQxNXTmBWboDPgV1PA/wg==',
        'sl_jwt_session': 'nBfxJQijBGbhq2HKUw56CQ==',
        'cdn': 'aitianhu',
        'SERVERID': 'srv99n3|ZgSVF',
    }

    headers = {
        'accept': 'application/json, text/plain, */*',
        'accept-language': 'es-ES,es;q=0.9',
        'content-type': 'application/json',
        'origin': 'https://715zwv.aitianhu1.top',
        'referer': 'https://715zwv.aitianhu1.top/',
        'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-origin',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
    }

    json_data = {
        'prompt': prompt,
        'options': {
            'parentMessageId': 'chatcmpl-97QRFUFrq5WLDuLsTTfUvUPYLaTQK',
        },
        'model': 'gpt-3.5-turbo',
        'OPENAI_API_KEY': 'sk-AItianhuFreeForEveryone',
        'systemMessage': "You are an AI assistant, a large language model trained. Follow the user's instructions carefully. Respond using markdown.",
        'temperature': 0.8,
        'top_p': 1,
    }

    response = requests.post(
        'https://715zwv.aitianhu1.top/api/please-donot-reverse-engineering-me-thank-you',
        cookies = cookies,
        headers = headers,
        json = json_data,
    )
    # The response is a serie of json elements in raw format having the response but
    # in a word-increase way. Each json has a new word. I only care about the last one
    response = response.content
    response = response.splitlines()
    response = json.loads(response[len(response) - 1])

    return response['text']

def ask_v2(prompt):
    """
    ** NOT WORKING **
    This is not working because of a rate limit. I need a 'sign' parameter
    that I don't know where it comes from, and I think that is the one that
    makes the request fail, as I can continue making requests through the
    website. Developed with Astro framework.

    TODO: Keep working to make it work
    """
    # From this: https://chatforai.store/
    import requests
    import datetime

    headers = {
        'Accept': '*/*',
        'Accept-Language': 'es-ES,es;q=0.9',
        'Connection': 'keep-alive',
        'Content-Type': 'text/plain;charset=UTF-8',
        'Origin': 'https://chatforai.store',
        'Referer': 'https://chatforai.store/',
        'Sec-Fetch-Dest': 'empty',
        'Sec-Fetch-Mode': 'cors',
        'Sec-Fetch-Site': 'same-origin',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
        'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
    }
    
    # TODO: I don't know where does this sign come from and I need it
    sign = 'b4eb4fd22a63c55c75faedb303507e04c4053d340e99820da157b5ee6f6b2c4b'
    timestamp = str(datetime.datetime.now().timestamp() * 1000)
    data = '{"conversationId":"f317d080-1aaf-414f-88bc-fdee7d4b1c51","conversationType":"chat_single","botId":"chat_single","globalSettings":{"baseUrl":"https://api.openai.com","model":"gpt-3.5-turbo","maxTokens":2048,"messageHistorySize":5,"temperature":0.7,"top_p":1},"prompt":"' + prompt + '","messages":[{"role":"user","content":"' + prompt + '"}],"sign":"' + sign + '","timestamp":' + timestamp + '}'
    data = data.encode()

    response = requests.post('https://chatforai.store/api/handle/provider-openai', headers=headers, data=data)

    print(response)
    print(response.text)
    print(response.content)
    return response.content

def ask_v3(prompt):
    """
    ** NOT WORKING **
    This seems to be working not because of Cloudflare detecting me. I
    don't know how as I can ask through the web page. Maybe is the 
    messages history.
    """
    # From here: https://flowgpt.com/chat
    import requests

    headers = {
        'accept': '*/*',
        'accept-language': 'es-ES,es;q=0.9',
        'authorization': 'Bearer null',
        'content-type': 'application/json',
        'origin': 'https://flowgpt.com',
        'referer': 'https://flowgpt.com/',
        'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-site',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
        'x-nonce': '',
        'x-signature': '',
        'x-timestamp': '',
    }

    json_data = {
        'model': 'gpt-3.5-turbo',
        'nsfw': False,
        'question': prompt,
        'history': [
            {
                'role': 'assistant',
                'content': 'Hello there 😃, I am ChatGPT. How can I help you today?',
            },
        ],
        'system': "You are help assitant. Follow the user's instructions carefully. Respond using markdown",
        'temperature': 0.7,
        'promptId': 'model-gpt-3.5-turbo',
        'documentIds': [],
        'chatFileDocumentIds': [],
        'generateImage': False,
        'generateAudio': False,
    }

    response = requests.post('https://backend-k8s.flowgpt.com/v2/chat-anonymous', headers=headers, json=json_data)
    # Force it to receive well-written response
    response.encoding = 'utf-8'
    response = response.content
    response = response.splitlines()
    print(response)
    answer_text = ''
    for response_line in response:
        try:
            response_line = json.loads(response_line)
            answer_text += response_line['data']
        except:
            pass

    return answer_text
    #response = json.loads(response[len(response) - 1])

    # Response comes like this 
    """
    {"event":"text","data":"Seg"}

    {"event":"text","data":"ún"}
    """

def ask_v4(prompt):
    """
    This is simple but is working. It seems to be limited to just 25 questions, and 
    is IP-based, so I'm done.
    """
    # From here: https://koala.sh/chat
    cookies = {
        'cf_clearance': 'raYChfDvF_OzTTdMcV19UIIJXxMPE2Y1k1nPH5Fhvn0-1711565095-1.0.1.1-YLtUJHtzM_qv3YVmzFOzN38MXvgiLJEbouvkAd2x3C8WAZ6GKSP6As3PpOOqutPH0GCMGthJojO_5_Xl7iH3bQ',
        '_iidt': 'k7gejbRstP+5a5IOxRdUUQf2RgGTp0vVUDPZDAQxtoTQ45MPQU7NtVDrbuoVlvC+MEJqH0L+pzwZlw==',
        '_vid_t': '0oU1V6DiFStM/i7nXc9Bf8AFH7REsgVV9MjIYwYt9Vcb2RZOcVtUaXgWND9vBu3m1P21fQA6xOT5wg==',
        '__stripe_mid': 'e9642a6d-8941-45a7-a7fa-18d5b155970df1843d',
        '__stripe_sid': 'e42f8f7f-cfee-48d0-83b0-846be8a59508405ab3',
    }

    headers = {
        'accept': 'text/event-stream',
        'accept-language': 'es-ES,es;q=0.9',
        'content-type': 'application/json',
        'flag-real-time-data': 'false',
        'origin': 'https://koala.sh',
        'referer': 'https://koala.sh/chat',
        'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-origin',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
        'visitor-id': 'vMXyv8qXyfcekvaCejnP',
    }

    json_data = {
        'input': prompt,
        'inputHistory': [],
        'outputHistory': [],
        'model': 'gpt-3.5-turbo',
    }

    response = requests.post('https://koala.sh/api/gpt/', headers=headers, json=json_data)
    response
    # Force it to receive well-written response
    response.encoding = 'utf-8'
    response = response.text.split('\n')
    accumulated_text = ''
    for text_line in response:
        if 'data' in text_line:
            accumulated_text += text_line.split(':')[1].strip().replace('"', '')
    return accumulated_text

    # Response is like this below:
    """
            
        data: " de"

        data: " personas"

        data: "."

    """

    # TODO: Continue watching below to get working endpoints:
    # https://github.com/xtekky/gpt4free?tab=readme-ov-file#-providers-and-models
    # https://github.com/zukixa/cool-ai-stuff

def ask_v5(prompt):
    """
    ** NOT WORKING **
    This seems to be working not because of Cloudflare detecting me. I
    don't know how as I can ask through the web page.
    Maybe this (https://stackoverflow.com/a/71404659) or a ChromeDriver instance
    """
    import requests

    headers = {
        'accept': 'application/json',
        'accept-language': 'es',
        'content-type': 'application/json',
        'myshell-client-version': 'v1.5.4',
        'myshell-security-token': 'zfIvFpxwEDNEY1QFABfA+IEGemNJyvDl',
        'myshell-service-name': 'organics-api',
        'origin': 'https://app.myshell.ai',
        'platform': 'web',
        'referer': 'https://app.myshell.ai/',
        'sc-cookie-id': '18e8172f5cd2f5-0d0cc84e196ca68-26001a51-1247616-18e8172f5ce96d',
        'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-site',
        'timestamp': '1711568924954',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
        'visitor-id': 'd56026c8-3772-46f2-92fe-840c12cff446',
    }

    json_data = {
        'botId': '4738',
        'message': prompt,
        'messageType': 1,
    }

    response = requests.post('https://api.myshell.ai/v1/bot/chat/send_message', headers=headers, json=json_data)
    response.encoding = 'utf-8'
    response = response.text.split('{')
    print(response)
    response = json.loads('{' + response[len(response) - 1].replace('\n', ''))

    return response['message']['text']

    # Response is like this below:
    """
    
        id: 5131ab92-82ff-44c4-aa8a-89bf52ca4ac9
        event: MESSAGE_REPLY_SSE_ELEMENT_EVENT_NAME_TEXT_STREAM_PUSH_FINISHED
        data: {"content":"", "voiceBytes":"", "voiceBytesIndex":0, "errorReason":"ERROR_REASON_UNSPECIFIED", "status":null, "voiceFilePath":"", "message":null, "audioFileDurationSeconds":0, "userEnergyInfo":null, "imageGenMessageResponse":null}

        id: 34a534ac-9e8f-4e3c-925d-1e2c3ca0e804
        event: MESSAGE_REPLY_SSE_ELEMENT_EVENT_NAME_USER_SENT_MESSAGE_REPLIED
        data: {"content":"", "voiceBytes":"", "voiceBytesIndex":0, "errorReason":"ERROR_REASON_UNSPECIFIED", "status":null, "voiceFilePath":"", "message":{"id":"147195724", "uid":"2965f0771f434eb9a6204b92f263d583", "userId":"8897689", "type":"REPLY", "botId":"4738", "replyUid":"5a26b39eabe0458b8efa74fc282e559e", "status":"DONE", "text":"Según el gobierno metropolitano de Tokio, la población de Tokio en 2021 es de aproximadamente 13,96 millones de habitantes, lo que la convierte en la ciudad más poblada de Japón y una de las más grandes del mundo.", "handled":false, "translation":"", "voiceUrl":"", "voiceFileDurationSeconds":0, "feedbackState":"Normal", "feedbackIssues":[], "createdDateUnix":"1711568926031", "updatedDateUnix":"1711568926031", "audioSpeed":1, "imageGenMessageResponse":null, "embedObjs":[], "imSlashCommandInput":null, "asyncJobInfo":null, "widgetId":"0", "replyId":"147195723", "referenceSource":[], "recommendationQuestion":{"question":[]}, "componentContainer":null, "inputSetting":null}, "audioFileDurationSeconds":0, "userEnergyInfo":null, "imageGenMessageResponse":null}


    """









"""
This below could be working not, sorry
"""

def ask_deprecated(prompt):
    """
    Sends a message through a GPT conversation (using the code-written conversation_id and session
    token). It returns the response.
    """
    # TODO: This is not working, but they are trying to fix it in the 
    # repo (https://github.com/Barrierml/reverse-engineered-chatgpt). 
    # I had to manually re-write 'sync_chatgpt.py' and 'async_chatgpt.py'
    # files but still not working.
    # I receive a 'Cannot concatenate ...' that seems to be something
    # with their code

    # Go to ChatGPT and obtain that Cookie (F12 + Application + Cookies + '__Secure-next-auth.session-token here'
    SESSION_TOKEN = "eyJhbGciOiJkaXIiLCJlbmMiOiJBMjU2R0NNIn0..uJDTNWyIWEV-Z299.WF6tfBGa1HFc6yU8KyFensoUf8KvnSyzvNk53-pmjD_x3qjiM4ubYzOL-XJhe1yt0RqHJjInig7GG0B1Y6906eCavPOfeevLPmGNlBJGkpBVkiXAhGvnRkQtWNVjzwMYpwF0-U7BzHoVXM0Lv90xm4DDtKnE4FmRWl814mJxToVXdeVGmkjij8yID_RGYE1Ibot5p-gGJQqVG_w6k9v-SopUpZHKaXQbndjj5ORA1rse5E1h_0oU3LPY58dOUud5zIndJ1vE_dsf2kZFYT0hmU_EbjF4qbLcGvpBMFjOaqPLM-FkssIfF4TuZ7svmbKbq53nGmSsFM5O_MGP1EJzY8Uwm3SwN31i8sfFCZhCgGn6Drvkjv0iA8xhGM0aEhqv4r0gAPJpW4bNuabSxpL5X3LGELgk7qSNZ9UFcENmyqv90lLq4SE_CUeQmXajt0JZz8DqmqlYVvIotbRbhq78AC9KS1b0l2Jz8bI-vKmzA7odbp2Juw8BFx4e3386At9lQwjM-vJHTQR1MsQrPse5eHxPak6qLdNHptaQf7XbpxgMEQrSkoki0E-VUtwy7QYtMKffZuL1Vg_SGQk4zrX3dFypH97hqvQ1azZu-dtdLEDO6AtgwUXvcSnmlO3bGUX4zSLFh3oUkAS98AMZgOPYuY4oMk5o8Q6P700hJhfzQgvek59OX__snqeTO3Sltc8R14N84UWqFYFcl0X6wonB09m4zZvgzy8qBNGgeoxAUF6LrV8RD2yAYvDytrqcP6QuIbNN6RvqgDVN3EYT3lGLLBa3higJKA_l6DWin3uB984_icNQ9p93P4BI61BW_uL8FHMcj1pZ4pyPA3gVmTINY1gfwc8-ObPgQW67VnL57shi3BD5ujdU6_gqNL53sw-96HwBnJXJKLwaxwF5t_I7fyTcKq-fx-qaYiYVpLrDL3g3JylZ-jElQDLN5gN_GJ0MkGo8gUC_T0XK6w4MoOdU60zVCWEFoJ10Q-qIEfNr3F-URlh-LnLt0SS2uDdjMvLdEbXj3A_lVjFVHoQmujSMs_E0GcAgSUb6zBvnzZO7yvUYwfyqGvJwbaCDxa9bq7Ec2bb1MKn3bqOJJvHRSBbzGYVVXshT4WUOlqh24hBP9hgaT4BwooGI9jhRw_rda0rAZ6IlvBazZQ33qZzVbc5W3nyxNUIzbhUICX96pTlCSPZcRlmtItxsCljLxrzHTsij6QQxtJ0ugr8IvFzTisaST5qQgZwLtgPSf7Y2NRkkY_16etKhEuAwFWLbm374xp475Wyj5RNNPF8BzS-N38JSjrH5iu8yvTEaB_VHYgVsUYiuHY-R4zkVyIAsULemqB6pki3PPIQzJkAY99INI72ZRcWdyZycs-jxUrVuZh35CJpYr9WbgTpwrpW9Kz6K6qnFWblsv507dz4sr5uWskXr-xqavvTPVQh2-_9idRmSa1YweMt9FVzNLLg_aJqMtYXqJf5z_iFcliVfTConsHWL0fpHjJyIyKZOje4m-SEk_2FlkdbRr2FqkTequNxS1ND2rj5wroR1p9BVybwRDiKNYrAQuGnZHltQicJzYPjFphCzdPKMfQGJRtAYCkNM-Nq-V3k3iZ4vF37H1PP-swxnp-neus2K_YuH6KeVOTKMK74F3HMJJLkuBmMn4D15iBcd4K8PBvaeVxZi9tsfFX0TElE2gJzwjLAc2kJQxZzwn-SwK5BSsKPLMMjoAO3XBnD2VwlplXV5nk92PSSFhl4-T8WetMEOn5QkJBOH36cbxSbCYgDOB90AbfhdpvAVgiX1mcqBSHp51ISbvXrerGWNkMrP20E0V_FHG5MCqwH1YvV2FKPo8PFC3gv96q60HrffwF96gu0v5OtqGtHR2k3aevwuNUqB4smXF04fd6tRJLyHq1fYu5PvyXYQrzSp27UVtuHAfxXb38KLw-Gdn95R1P_LBps9OTU71BPkYZwbAA69UaPk4R5rkX832tGHMTM5HOE8ChSQk-pfaPzuE6UpXJ0nh4EncyKQCYl57oY0B1JSc_-WEmOlYhM6F5Cata_lkuC_JUlAaEsgGNosfW1qhksbSgSlQMQEmLCHP7p-eA4-UQnkgb1OR2sxdW7qCnXKICKiEftanAgYZbB6ha2c9xsHMVClkk-2Nui5XhSVx5Bo-QlZnjp0p5c7Xw-JDLY1uwmmoXYCaCKpLMJZkXpg_q8p-H5gR-jAoPoCsw3ajLGrkm36ubbKPc9_Nn3KQzXHg9GWxf35OeSu6CVvJAy7WGvtnlwnLnBlURMt9nRSgn341UN-jsyWrdnQQLu0ygwMDQMaJF-_M_8HxfmerssdT810Z3MGuhX14_2CMQ8pTI8fj_GUwe47NIj9OOpX697F-kzUMk_Q6SPCmJjPoPCoC2e4Z_f1ef8macrFJW5q_qMcBsOauDl5pPZvjkXCi-dlBBacAvdTKzO45dXPD7bxkOVJnok9kITp2VgVz5kr6HYLhsq98aM1SM_CZSdKDJaatokSZPP_JSYIOfnZdnqnnzIWQJXjZTTnTIFoGGR_AR8tokVptlpUszJUHnAtltdNi5B6mdsleeg4LJzpToHwMzd0iAn2TOsXgAwtdIcib7K9ppEk5mvj9oF3QAPkegEB83skfUAQGQSEBgtRXG6jjEm6on8WuaYRU8nrkSCiIQ2Opv-cWaaSfVDAm4t_Zz6ln59rBmLcVPpyi9CYsNhoGyReClj6hnrRhK6hciwnrVOMtoQ9T1LyfdGsBE4ZEX_AqDMfqO_pz58yvFFzMduRPVO-y8g2YVMNkH_R9R_qRXCHtTlJPcn2TOi9wuDf-ZK1WDLXBVV02wI2NOloVFGYTLvhJqnG6mEv3YkdGbIAsREWxUF01g.YcvVAzSNW20DDyUDL313Jw"
    CONVERSATION_ID = "31ca0c4a-77a0-4f1a-9161-9fd38dfe1afd"
    
    with SyncChatGPT(session_token = SESSION_TOKEN) as chatgpt:
        if CONVERSATION_ID:
            conversation = chatgpt.get_conversation(CONVERSATION_ID)
        else:
            # TODO: This below is untested, so I would store that new conversation_id I think
            conversation = chatgpt.create_new_conversation()

        response = ""

        for message in conversation.chat(prompt):
            response += message['content']

        return response
    
def get_image_generation_prompt(idea):
    """
    Returns the GPT response about a nice midjourney prompt to represent the provided 'idea'.
    """
    return __ask_gpt(__get_image_generation_prompt_input(idea))

def get_stock_video_keywords(idea):
    """
    Returns the GPT response about nice stock video keywords to look for the perfect video that
    fits the provided 'idea'.
    """
    return __ask_gpt(__get_stock_video_input(idea))

def get_feeling_from_input(input):
    """
    Returns the GPT response about the feeling that the provided 'input' transmits.
    """
    return __ask_gpt(__get_feeling_from_input(input))

def get_topic_from_input(input):
    """
    Returns the GPT response about the main topic of the provided 'input' talks about.
    """
    return __ask_gpt(__get_topic_from_input(input))

def get_translation(input, input_language = 'inglés', output_language = 'español'):
    """
    Returns the provided 'input' English text translated into Spanish.
    """
    return __ask_gpt(__get_translate_text_input(input, input_language, output_language))

def test(input):
    """
    This method is for testing. Asks GPT the 'input' and returns its response.
    """
    return __ask_gpt(input)

"""
    ¿Habla de personas o de cosas?
    '¿Me puedes decir si la frase "esto es una casa, no hay más que verlo" está hablando sobre personas o sobre cosas? Solamente respóndeme "personas" si está hablando de personas, o "cosas" si está hablando de cosas.')

    ¿Tema de la frase?
"""



from moviepy.editor import ImageSequenceClip, AudioFileClip, VideoFileClip, TextClip, CompositeVideoClip
from moviepy.video.tools.subtitles import SubtitlesClip
from tqdm import tqdm
from yta_general_utils.text_processor import remove_accents
from yta_voice_module.stt.whisper import get_transcription_with_timestamps

import whisper_timestamped as whisper
import os
import cv2

def subtitle(video_filename):
    """
    Receives a video and creates a new one with subtitles at the bottom.
    """
    video = VideoFileClip(video_filename)
    video.audio.write_audiofile('tmp_transcription.mp3')
    transcription = get_transcription_with_timestamps('tmp_transcription.mp3')

    generator = lambda txt: TextClip(txt,
        font = 'Ubuntu-bold',
        fontsize = 72,
        color = 'white',
        stroke_color = 'black',
        stroke_width = 1,
    )

    subtitles = []
    for element in transcription['segments'][0]['words']:
        subtitles.append(((element['start'], element['end']), element['text']))

    subtitles = SubtitlesClip(subtitles, generator)
    result = CompositeVideoClip([video, subtitles.set_pos(('bottom'))])
    result.write_videofile("video_subtitled.mp4",
        fps = video.fps,
        temp_audiofile = "temp-audio.m4a",
        remove_temp = True,
        codec = "libx264",
        audio_codec = "aac"
    )

def transcribe(audio_filename):
    """
    TODO: This is being tested
    TEST: Obtains the subtitles based on video audio and sets it. Working on
    test files, not actual files.
    """
    # Testing video
    video = VideoFileClip('test_transcription.mp4') # 2.
    audio = video.audio # 3.
    audio.write_audiofile('test_transcription.mp3') # 4.

    transcription = get_transcription_with_timestamps('test_transcription.mp3')

    generator = lambda txt: TextClip(txt,
        font = 'Ubuntu-bold',
        fontsize = 72,
        color = 'white',
        stroke_color = 'black',
        stroke_width = 1,
    )

    subtitles = []
    for element in transcription['segments'][0]['words']:
        subtitles.append(((element['start'], element['end']), element['text']))

    subtitles = SubtitlesClip(subtitles, generator)
    video = VideoFileClip("test_transcription.mp4")
    # TODO: Here we have the audio error again
    result = CompositeVideoClip([video, subtitles.set_pos(('center'))])
    result.write_videofile("output.mp4",
        fps = video.fps,
        temp_audiofile = "temp-audio.m4a",
        remove_temp = True,
        codec = "libx264",
        audio_codec = "aac"
    )
    
def detect_word(audio_filename, word):
    """
    This method makes a transcription of the provided 'audio_filename',
    iterates over all words found and detects if the provided 'word' is
    in that audio. It returns an array with each time that 'word' exist
    in the audio, telling the 'start' and 'end' of that word in the 
    audio.
    """
    found = []

    transcription = get_transcription_with_timestamps(audio_filename)
    for element in transcription['segments'][0]['words']:
        if word in remove_accents(element['text'].lower()):
            found.append({
                'word': word,
                'start': element['start'],
                'end': element['end']
            })

    return found

# TODO: Remove this below as above is working
class VideoTranscriber:
    def __init__(self, model_path, video_path):
        self.model = whisper.load_model(model_path)
        self.video_path = video_path
        self.audio_path = ''
        self.text_array = []
        self.fps = 0
        self.char_width = 0

    def transcribe_video(self):
        print('Transcribing video')
        result = self.model.transcribe(self.audio_path)
        text = result["segments"][0]["text"]
        textsize = cv2.getTextSize(text, cv2.FONT_HERSHEY_SIMPLEX, 0.8, 2)[0]
        cap = cv2.VideoCapture(self.video_path)
        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        asp = 16/9
        ret, frame = cap.read()
        width = frame[:, int(int(width - 1 / asp * height) / 2):width - int((width - 1 / asp * height) / 2)].shape[1]
        width = width - (width * 0.1)
        self.fps = cap.get(cv2.CAP_PROP_FPS)
        self.char_width = int(textsize[0] / len(text))
        
        for j in tqdm(result["segments"]):
            lines = []
            text = j["text"]
            end = j["end"]
            start = j["start"]
            total_frames = int((end - start) * self.fps)
            start = start * self.fps
            total_chars = len(text)
            words = text.split(" ")
            i = 0
            
            while i < len(words):
                words[i] = words[i].strip()
                if words[i] == "":
                    i += 1
                    continue
                length_in_pixels = len(words[i]) * self.char_width
                remaining_pixels = width - length_in_pixels
                line = words[i] 
                
                while remaining_pixels > 0:
                    i += 1 
                    if i >= len(words):
                        break
                    length_in_pixels = len(words[i]) * self.char_width
                    remaining_pixels -= length_in_pixels
                    if remaining_pixels < 0:
                        continue
                    else:
                        line += " " + words[i]
                
                line_array = [line, int(start) + 15, int(len(line) / total_chars * total_frames) + int(start) + 15]
                start = int(len(line) / total_chars * total_frames) + int(start)
                lines.append(line_array)
                self.text_array.append(line_array)
        
        cap.release()
        print('Transcription complete')
    
    def extract_audio(self, output_audio_path='audio.mp3'):
        print('Extracting audio')
        video = VideoFileClip(self.video_path)
        audio = video.audio 
        audio.write_audiofile(output_audio_path)
        self.audio_path = output_audio_path
        print('Audio extracted')
    
    def extract_frames(self, output_folder):
        print('Extracting frames')
        cap = cv2.VideoCapture(self.video_path)
        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        asp = width / height
        N_frames = 0
        
        while True:
            ret, frame = cap.read()
            if not ret:
                break
            
            frame = frame[:, int(int(width - 1 / asp * height) / 2):width - int((width - 1 / asp * height) / 2)]
            
            for i in self.text_array:
                if N_frames >= i[1] and N_frames <= i[2]:
                    text = i[0]
                    text_size, _ = cv2.getTextSize(text, cv2.FONT_HERSHEY_SIMPLEX, 0.8, 2)
                    text_x = int((frame.shape[1] - text_size[0]) / 2)
                    text_y = int(height/2)
                    cv2.putText(frame, text, (text_x, text_y), cv2.FONT_HERSHEY_SIMPLEX, 0.75, (0, 0, 255), 2)
                    break
            
            cv2.imwrite(os.path.join(output_folder, str(N_frames) + ".jpg"), frame)
            N_frames += 1
        
        cap.release()
        print('Frames extracted')

    def create_video(self, output_video_path):
        print('Creating video')
        image_folder = os.path.join(os.path.dirname(self.video_path), "frames")
        if not os.path.exists(image_folder):
            os.makedirs(image_folder)
        
        self.extract_frames(image_folder)
        
        print("Video saved at:", output_video_path)
        images = [img for img in os.listdir(image_folder) if img.endswith(".jpg")]
        images.sort(key=lambda x: int(x.split(".")[0]))
        
        frame = cv2.imread(os.path.join(image_folder, images[0]))
        height, width, layers = frame.shape
        
        clip = ImageSequenceClip([os.path.join(image_folder, image) for image in images], fps=self.fps)
        audio = AudioFileClip(self.audio_path)
        clip = clip.set_audio(audio)
        clip.write_videofile(output_video_path)

"""
# Example usage
model_path = "base"
video_path = PROJECT_ABSOLUTE_PATH + "output/2024-01-18_16-34-50.mp4"
output_video_path = PROJECT_ABSOLUTE_PATH + "output/transcription.mp4"

transcriber = VideoTranscriber(model_path, video_path)
transcriber.extract_audio()
transcriber.transcribe_video()
transcriber.create_video(output_video_path)
"""


from setuptools import setup, find_packages


VERSION = '0.0.5'
DESCRIPTION = 'Youtube Autónomo AI utils are here.'
LONG_DESCRIPTION = 'These are the AI utils we need in the Youtube Autónomo project to work in a better way.'

setup(
    name = "yta_ai_utils", 
    version = VERSION,
    author = "Daniel Alcalá",
    author_email = "<danielalcalavalera@gmail.com>",
    description = DESCRIPTION,
    long_description = LONG_DESCRIPTION,
    packages = find_packages(),
    install_requires = [
        'yta-general-utils',
        'google-generativeai',
        'faster_whisper',
        'whisper_timestamped',
    ],
    
    keywords = [
        'youtube autonomo ai utils'
    ],
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


from yta_general_utils.downloader import Downloader
from yta_general_utils.dataclasses import FileReturn
from yta_general_utils.programming.output import Output
from yta_general_utils.file.enums import FileTypeX
from yta_general_utils.programming.parameter_validator import PythonValidator
from yta_general_utils.url import encode_url_parameter
from typing import Union


class Pollinations:
    """
    Class to wrap prodia functionality. Prodia is an
    image generator engine.
    """

    @staticmethod
    def generate_image(
        prompt: str,
        output_filename: Union[str, None] = None
    ) -> FileReturn:
        if not PythonValidator.is_string(prompt):
            raise Exception('The provided "prompt" is not a valid string.')
        
        output_filename = Output.get_filename(output_filename, FileTypeX.IMAGE)
        
        prompt = encode_url_parameter(prompt)

        # TODO: Make some of these customizable
        WIDTH = 1920
        HEIGHT = 1080
        # TODO: This seed should be a random value or
        # I will receive the same image with the same
        # prompt
        SEED = 43
        MODEL = 'flux'

        url = f'https://pollinations.ai/p/{prompt}?width={WIDTH}&height={HEIGHT}&seed={SEED}&model={MODEL}'

        return Downloader.download_image(url, output_filename)

"""
    Check because there is also a model available for
    download and to work with it (as they say here
    https://pollinations.ai/):

    # Using the pollinations pypi package
    ## pip install pollinations

    import pollinations as ai

    model_obj = ai.Model()

    image = model_obj.generate(
        prompt=f'Awesome and hyperrealistic photography of a vietnamese woman... {ai.realistic}',
        model=ai.flux,
        width=1038,
        height=845,
        seed=43
    )
    image.save('image-output.jpg')

    print(image.url)
    """